package example.websocket;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.ByteBuffer;

import javax.websocket.DeploymentException;
import javax.websocket.Endpoint;
import javax.websocket.EndpointConfig;
import javax.websocket.SendHandler;
import javax.websocket.SendResult;
import javax.websocket.Session;

import example.websocket.factory.ClientEndpointFactory;

public abstract class EndpointBase extends Endpoint implements WebsocketEndpoint{
	private Session session = null;
	private boolean runing = true;
	protected long minimum = 5000; 
	protected long maximum = 15000;
	
	public EndpointBase() throws DeploymentException, IOException, URISyntaxException {
		ClientEndpointFactory.getInstance().connect(this);
	}
	
	@Override
	final public void run() {
		if(session == null){
			try {
				wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		while(runing){
			alive();
			try {
				Thread.sleep(minimum + (int)(Math.random() * maximum));
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	public abstract void alive();
	public abstract void onMessage(Session session , String message);
	
	@Override
	final public void onOpen(Session arg0, EndpointConfig arg1) {
		session = arg0;
		notify();
	}
	
	final public void sendMessage(String message){
		//session.getAsyncRemote().sendText(message);
		session.getAsyncRemote().sendText(message, new SendHandler() {
			
			@Override
			public void onResult(SendResult arg0) {
				String x = "";
			}
		});
	}
	final public void sendMessage(ByteBuffer buffer){
		session.getAsyncRemote().sendBinary(buffer);
	}
	final public void sendMessage(Object obj){
		session.getAsyncRemote().sendObject(obj);
	}
	
	final public void shutdown(){
		this.runing = false;
	}
}
